@@ -7,15 +7,19 @@ use std::borrow::Cow;
 use std::collections::HashMap;
 
 use anyhow::Context;
-use k8s_openapi::api::core::v1::{ConfigMap, Namespace, Node, Pod, Secret};
+use k8s_openapi::api::apps::v1 as appsv1;
+use k8s_openapi::api::core::v1 as corev1;
+use k8s_openapi::apiextensions_apiserver::pkg::apis::apiextensions::v1 as apiextensionsv1;
 use kube::api::{self, Api};
 use kube::config::{Config, KubeConfigOptions, Kubeconfig};
 use kube::{Client, Resource, ResourceExt};
 use serde_json as json;
+use statehub_cluster_operator::StatehubCluster;
 
 use crate::location::Location;
 use crate::v0;
 
+pub(crate) use self::diagnose::Probe;
 pub(crate) use self::helm::Helm;
 
 use self::helper::{group_nodes_by_region, group_nodes_by_zone, is_aks, is_eks, NamesAndContexts};
@@ -32,10 +36,6 @@ mod show;
 pub(crate) const KUBE_SYSTEM_NS: &str = "kube-system";
 const STATEHUB_CLUSTER_TOKEN_SECRET_TYPE: &str = "statehub.io/cluster-token";
 const STATEHUB_CLUSTER_TOKEN_SECRET_NAME: &str = "statehub-cluster-token";
-const STATEHUB_CLOUD_CREDS_SECRET_TYPE: &str = "statehub.io/cloud-credential";
-const STATEHUB_AZURE_CREDS_SECRET_NAME: &str = "statehub-azure-creds";
-const STATEHUB_AWS_CREDS_SECRET_NAME: &str = "statehub-aws-creds";
-const STATEHUB_CLUSTER_CONFIGMAP_NAME: &str = "statehub";
 
 #[derive(Debug)]
 pub(crate) struct Kubectl {
@@ -163,6 +163,14 @@ impl Kubectl {
         }
     }
 
+    pub(crate) async fn get_operator(
+        &self,
+    ) -> anyhow::Result<statehub_cluster_operator::cluster::Operator> {
+        let config = self.get_config().await?;
+        let operator = statehub_cluster_operator::cluster::Operator::with_config(config)?;
+        Ok(operator)
+    }
+
     async fn get_config(&self) -> anyhow::Result<Config> {
         let context = Some(self.context.clone());
         let options = KubeConfigOptions {
@@ -182,95 +190,27 @@ impl Kubectl {
             .context("Connecting to Kubernetes cluster")
     }
 
-    async fn all_namespaces(&self) -> anyhow::Result<impl IntoIterator<Item = Namespace>> {
+    async fn all_namespaces(&self) -> anyhow::Result<impl IntoIterator<Item = corev1::Namespace>> {
         let namespaces = self.api().await?;
         let lp = self.list_params();
         Ok(namespaces.list(&lp).await?)
     }
 
-    async fn all_nodes(&self) -> anyhow::Result<impl IntoIterator<Item = Node>> {
+    async fn all_nodes(&self) -> anyhow::Result<impl IntoIterator<Item = corev1::Node>> {
         let nodes = self.api().await?;
         let lp = self.list_params();
         Ok(nodes.list(&lp).await?)
     }
 
-    async fn all_pods(&self, namespace: &str) -> anyhow::Result<impl IntoIterator<Item = Pod>> {
+    async fn all_pods(
+        &self,
+        namespace: &str,
+    ) -> anyhow::Result<impl IntoIterator<Item = corev1::Pod>> {
         let pods = self.namespaced_api(namespace).await?;
         let lp = self.list_params();
         Ok(pods.list(&lp).await?)
     }
 
-    async fn create_configmap(
-        &self,
-        name: &str,
-        namespace: &str,
-        cluster_name: &v0::ClusterName,
-        default_state: &str,
-        api: &str,
-        cleanup_grace: &str,
-    ) -> anyhow::Result<ConfigMap> {
-        let configmaps = self.namespaced_api(namespace).await?;
-        let configmap = json::from_value(json::json!({
-            "apiVerion": "v1",
-            "kind": "ConfigMap",
-            "metadata": {
-                "name": name,
-                "namespace": namespace,
-            },
-            "data": {
-                "cluster-name": cluster_name,
-                "default-state": default_state,
-                "api-url": api,
-                "cleanup-grace": cleanup_grace,
-            }
-        }))?;
-        let pp = self.post_params();
-        let configmap = configmaps.create(&pp, &configmap).await?;
-
-        Ok(configmap)
-    }
-
-    pub(crate) async fn store_azure_credentials(
-        &self,
-        namespace: &str,
-        azure: json::Value,
-    ) -> anyhow::Result<Secret> {
-        self.set_secret(
-            namespace,
-            STATEHUB_CLOUD_CREDS_SECRET_TYPE,
-            STATEHUB_AZURE_CREDS_SECRET_NAME,
-            azure,
-        )
-        .await
-    }
-
-    pub(crate) async fn store_aws_credentials(
-        &self,
-        namespace: &str,
-        aws: json::Value,
-    ) -> anyhow::Result<Secret> {
-        self.set_secret(
-            namespace,
-            STATEHUB_CLOUD_CREDS_SECRET_TYPE,
-            STATEHUB_AWS_CREDS_SECRET_NAME,
-            aws,
-        )
-        .await
-    }
-
-    async fn delete_configmap(&self, namespace: &str, configmap: &str) -> anyhow::Result<()> {
-        log::info!("Deleting configmap {}", configmap);
-        let configmaps = self.namespaced_api::<ConfigMap>(namespace).await?;
-        let dp = self.delete_params();
-        configmaps
-            .delete(configmap, &dp)
-            .await?
-            .map_left(|cm| log::info!("Delete in progress {:#?}", cm))
-            .map_right(|cm| log::info!("Delete succeeded: {:#?}", cm));
-
-        Ok(())
-    }
-
     fn delete_params(&self) -> api::DeleteParams {
         api::DeleteParams::default()
     }
@@ -332,33 +272,6 @@ impl Kubectl {
             .map_err(anyhow::Error::msg)
     }
 
-    pub(crate) async fn store_configmap(
-        &self,
-        namespace: &str,
-        cluster_name: &v0::ClusterName,
-        default_state: &str,
-        api: &str,
-        cleanup_grace: &str,
-    ) -> anyhow::Result<ConfigMap> {
-        if self
-            .delete_configmap(namespace, STATEHUB_CLUSTER_CONFIGMAP_NAME)
-            .await
-            .is_ok()
-        {
-            log::trace!("Removing previous configmap");
-        }
-
-        self.create_configmap(
-            STATEHUB_CLUSTER_CONFIGMAP_NAME,
-            namespace,
-            cluster_name,
-            default_state,
-            api,
-            cleanup_grace,
-        )
-        .await
-    }
-
     pub(crate) async fn ensure_statehub_cluster_is_not_installed(&self) -> anyhow::Result<()> {
         if let Ok(cluster) = self.check_statehub_cluster_object().await {
             anyhow::bail!(
@@ -369,17 +282,22 @@ impl Kubectl {
         Ok(())
     }
 
-    pub(crate) async fn install_statehub_cluster(&self, namespace: &str) -> anyhow::Result<()> {
+    pub(crate) async fn install_statehub_cluster(
+        &self,
+        namespace: &str,
+    ) -> anyhow::Result<StatehubCluster> {
         let crd = self.create_statehub_cluster_crd().await?;
         log::debug!("Created StatehubCluster CRD: {:#?}", crd);
         let cluster = self.create_statehub_cluster(namespace).await?;
         log::debug!("Created StatehubCluster: {:#?}", cluster);
 
-        let config = self.get_config().await?;
-        let statehub_kubectl = statehub_cluster_operator::cluster::Operator::with_config(config)?;
-        let namespace = statehub_kubectl.ensure_namespace_exists(&cluster).await?;
+        let namespace = self
+            .get_operator()
+            .await?
+            .ensure_namespace_exists(&cluster)
+            .await?;
         log::debug!("Created namespace: {:#?}", namespace);
-        Ok(())
+        Ok(cluster)
     }
 
     pub(crate) async fn uninstall_statehub_cluster(&self) -> anyhow::Result<()> {
@@ -425,7 +343,7 @@ impl Kubectl {
         &self,
         namespace: &str,
         token: &str,
-    ) -> anyhow::Result<Secret> {
+    ) -> anyhow::Result<corev1::Secret> {
         let token = json::json!({ "cluster-token": token });
 
         self.set_secret(
@@ -439,20 +357,22 @@ impl Kubectl {
 
     pub(crate) async fn list_namespaces(
         &self,
-    ) -> anyhow::Result<impl IntoIterator<Item = Namespace>> {
+    ) -> anyhow::Result<impl IntoIterator<Item = corev1::Namespace>> {
         self.all_namespaces().await
     }
 
-    pub(crate) async fn list_nodes(&self) -> anyhow::Result<impl IntoIterator<Item = Node>> {
+    pub(crate) async fn list_nodes(
+        &self,
+    ) -> anyhow::Result<impl IntoIterator<Item = corev1::Node>> {
         self.all_nodes().await
     }
 
-    pub(crate) async fn list_pods(&self) -> anyhow::Result<impl IntoIterator<Item = Pod>> {
+    pub(crate) async fn list_pods(&self) -> anyhow::Result<impl IntoIterator<Item = corev1::Pod>> {
         self.all_pods(KUBE_SYSTEM_NS).await
     }
 }
 
-pub(crate) fn extract_cluster_token(secret: &Secret) -> Option<Cow<'_, str>> {
+pub(crate) fn extract_cluster_token(secret: &corev1::Secret) -> Option<Cow<'_, str>> {
     secret
         .data
         .as_ref()
