@@ -69,7 +69,8 @@ impl Helm {
             .iter()
             .map(|helm| {
                 let mut cmd = Command::new("helm");
-                cmd.arg("install")
+                cmd.arg("upgrade")
+                    .arg("--install")
                     .arg(&helm.chart)
                     .arg("--namespace")
                     .arg(namespace)
